from marshmallow import Schema, fields, validates_schema, ValidationError
from marshmallow_enum import EnumField

from portal.database.models import ProjectState, SubmissionState

"""
Schemas used to serialize/deserialize and validate of the models in the DB
"""


class UserSchema(Schema):
    """User schema
    """
    id = fields.Str(dump_only=True, required=True)
    uco = fields.Int(required=True)
    email = fields.Email(required=True)
    username = fields.Str()
    name = fields.Str()
    is_admin = fields.Bool(default=False, missing=False)
    submissions = fields.Nested('portal.rest.schemas.SubmissionSchema',
                                only=('id', 'note', 'state'),
                                many=True)
    review_items = fields.Nested('portal.rest.schemas.ReviewItemSchema', only=('id',), many=True)
    roles = fields.Nested('portal.rest.schemas.RoleSchema', only=('id', 'name', 'course.id'), many=True)
    groups = fields.Nested('portal.rest.schemas.GroupSchema', only=('id', 'name', 'course.id'), many=True)


class PasswordChangeSchema(Schema):
    """Password Change Schema
    """
    old_password = fields.String()
    new_password = fields.String(required=True)


class CourseSchema(Schema):
    """Course Schema
    """
    id = fields.Str(dump_only=True)
    name = fields.Str(required=True)
    codename = fields.Str(required=True)
    roles = fields.Nested('portal.rest.schemas.RoleSchema',
                          only=('id', 'name', 'description'),
                          many=True)
    groups = fields.Nested('portal.rest.schemas.GroupSchema', only=('id', 'name'), many=True)
    projects = fields.Nested('portal.rest.schemas.ProjectSchema', only=('id', 'name'), many=True)


class SubmissionFileParamsSchema(Schema):
    """Submission File Params Schema
    """
    source = fields.Nested('portal.rest.schemas.SubmissionFileSourceSchema')


class SubmissionFileSourceSchema(Schema):
    """Submissions File Source Schema
    """
    type = fields.Str()
    url = fields.Str()  # git@gitlab.fi.muni.cz:foo/project.git
    branch = fields.Str()
    checkout = fields.Str()

    @validates_schema
    def validate_present_fields(self, data):
        if data.get('type') is None:
            raise ValidationError(
                f"Submission source type is required. Supported values: 'git', 'zip'.")
        if data['type'] not in ('git', 'zip'):
            raise ValidationError(
                f"Submission source unsupported: {data['type']}. Supported sources: git, zip.")
        if data['type'] == 'git':
            if any(key not in data.keys() for key in ['url', 'branch', 'checkout']):
                raise ValidationError(
                    f"File source 'git' requires precisely 'url', "
                    f"'branch' and 'checkout' attributes."
                    )
        if data['type'] == 'zip':
            if any(key in data.keys() for key in ['url', 'branch', 'checkout']):
                raise ValidationError(f"File source 'zip' does not require any other attributes.")


class SubmissionCreateSchema(Schema):
    """Submission Create Schema
    """
    file_params = fields.Nested('portal.rest.schemas.SubmissionFileParamsSchema')
    project_params = fields.Dict()


class ProjectSchema(Schema):
    """Project Schema
    """
    id = fields.Str(dump_only=True)
    name = fields.Str(required=True)
    config = fields.Nested('portal.rest.schemas.ProjectConfigSchema',
                           exclude=('id', '_submissions_allowed_from',
                                    '_submissions_allowed_to', '_archive_from'))
    state = EnumField(ProjectState)
    course = fields.Nested(CourseSchema, only=('id', 'name', 'codename'))
    submissions = fields.Nested('portal.rest.schemas.SubmissionSchema', many=True,
                                only=('id', 'note', 'state', 'user'))

    groups = fields.Nested('portal.rest.schemas.GroupSchema', many=True, only=('id', 'name'))


class ProjectConfigSchema(Schema):
    """Project config Schema
    """
    id = fields.Str(dump_only=True)
    project = fields.Nested('portal.rest.schemas.ProjectSchema', only=('id', 'name'))
    test_files_source = fields.Str()
    file_whitelist = fields.Str()
    pre_submit_script = fields.Str()
    post_submit_script = fields.Str()
    submission_parameters = fields.Str()
    submission_scheduler_config = fields.Str()
    submissions_allowed_from = fields.LocalDateTime()  # or just DateTime?
    submissions_allowed_to = fields.LocalDateTime()
    archive_from = fields.LocalDateTime()


class RoleSchema(Schema):
    """Role Schema
    """
    id = fields.Str(dump_only=True)
    name = fields.Str()
    description = fields.Str()
    users = fields.Nested(UserSchema, only=('id', 'uco', 'email', 'username'), many=True)
    course = fields.Nested(CourseSchema, only=('id', 'name', 'codename'))
    # this could be done prettier (e.g. flattening the structure, showing only set fields)
    permissions = fields.Nested('portal.rest.schemas.RolePermissionsSchema')


class RolePermissionsSchema(Schema):
    """Role Permissions Schema
    """
    id = fields.Str(dump_only=True)
    role = fields.Nested(RoleSchema, only=('id', 'name', 'description'))
    view_course_limited = fields.Bool()
    view_course_full = fields.Bool()
    update_course = fields.Bool()
    handle_notes_access_token = fields.Bool()

    assign_roles = fields.Bool()
    write_roles = fields.Bool()
    read_roles = fields.Bool()

    write_groups = fields.Bool()
    read_groups = fields.Bool()

    write_projects = fields.Bool()
    read_projects = fields.Bool()
    archive_projects = fields.Bool()

    create_submissions = fields.Bool()
    resubmit_submissions = fields.Bool()
    evaluate_submissions = fields.Bool()

    read_submissions_all = fields.Bool()
    read_submissions_groups = fields.Bool()
    read_submissions_own = fields.Bool()
    read_all_submission_files = fields.Bool()

    read_reviews_all = fields.Bool()
    read_reviews_groups = fields.Bool()
    read_reviews_own = fields.Bool()

    write_reviews_all = fields.Bool()
    write_reviews_group = fields.Bool()
    write_reviews_own = fields.Bool()


class GroupSchema(Schema):
    """Group Schema
    """
    id = fields.Str(dump_only=True)
    name = fields.Str()
    course = fields.Nested(CourseSchema, only=('id', 'codename'))
    users = fields.Nested(UserSchema, only=('id', 'uco', 'email', 'username'), many=True)
    projects = fields.Nested(ProjectSchema, only=('id', 'name'), many=True)


class SubmissionSchema(Schema):
    """Submission Schema
    """
    id = fields.Str(dump_only=True)
    note = fields.Str()
    state = EnumField(SubmissionState)
    scheduled_for = fields.LocalDateTime()
    parameters = fields.Dict()
    project = fields.Nested(ProjectSchema, only=('id', 'name', 'course.id', 'course.codename'))
    user = fields.Nested(UserSchema, only=('id', 'username', 'uco', 'name', 'email'))


class ReviewSchema(Schema):
    """Review Schema
    """
    id = fields.Str(dump_only=True)
    submission = fields.Nested(SubmissionSchema, only=('id', 'note', 'state'))
    review_items = fields.Nested('portal.rest.schemas.ReviewItemSchema', many=True)


class ReviewItemSchema(Schema):
    """Review Item Schema
    """
    id = fields.Str(dump_only=True)
    content = fields.Str()
    file = fields.Str()
    line = fields.Int()
    review = fields.Nested(ReviewSchema, only=('id',))
    user = fields.Nested(UserSchema, only=('id', 'uco', 'email', 'username'))


class ComponentSchema(Schema):
    """Component Schema
    """
    id = fields.Str(dump_only=True)
    name = fields.Str()
    access_token = fields.Str()
    ip_address = fields.Str()
    type = fields.Str()
    notes = fields.Str()


class CourseImportConfigSchema(Schema):
    """Course Import Config Schema
    """
    roles = fields.Str()
    groups = fields.Str()
    projects = fields.Str()


class CourseImportSchema(Schema):
    """Course Import Schema
    """
    source_course = fields.Str(required=True)
    config = fields.Nested(CourseImportConfigSchema, required=True)


class UserListUpdateSchema(Schema):
    """User List Update Schema
    """
    add = fields.List(fields.Str())
    remove = fields.List(fields.Str())


class GroupImportSchema(Schema):
    """Group Import Schema
    """
    source_course = fields.Str()
    source_group = fields.Str()
    with_users = fields.Str()


class NotificationSendToSchema(Schema):
    """Notification Send To Schema
    """
    role = fields.Str()
    users = fields.List(fields.Str())
    group = fields.Str()
    course = fields.Str()


class NotificationSchema(Schema):
    """Notification Schema
    """
    data = fields.Dict(required=True)
    send_to = fields.Nested(NotificationSendToSchema, required=True)


class SubmissionResultTemplateSchema(Schema):
    """Submission Result Template Schema
    """
    template = fields.Str(required=True)
    role = fields.Str()
    user = fields.Str()


class SubmissionResultSchema(Schema):
    """Submission Result Schema
    """
    data = fields.Dict(required=True)
    templates = fields.List(
        fields.Nested(SubmissionResultTemplateSchema, required=True)
        )


user_schema = UserSchema(strict=True)
user_schema_reduced = UserSchema(strict=True, only=('id', 'username', 'uco', 'email', 'name'))
users_schema = UserSchema(many=True, only=('id', 'username', 'uco', 'email'))
password_change_schema = PasswordChangeSchema()
submission_schema = SubmissionSchema()
submissions_schema = SubmissionSchema(many=True,
                                      only=('id', 'state', 'project', 'scheduled_for', 'user'))
submission_create_schema = SubmissionCreateSchema()
submission_state_schema = SubmissionSchema(only=('state',))
reviews_schema = ReviewSchema(many=True)
review_schema = ReviewSchema()
role_schema = RoleSchema(strict=True)
roles_schema = RoleSchema(many=True, only=('id', 'name', 'description', 'course'))
permissions_schema = RolePermissionsSchema()
course_schema = CourseSchema(strict=True)
course_schema_reduced = CourseSchema(strict=True, only=('id', 'name', 'codename',))
courses_schema = CourseSchema(many=True, only=('id', 'name', 'codename'))
course_import_schema = CourseImportSchema(strict=True)
group_schema = GroupSchema(strict=True)
groups_schema = GroupSchema(many=True, only=('id', 'name', 'course'))
project_schema = ProjectSchema(strict=True)
projects_schema = ProjectSchema(many=True, only=('id', 'name', 'course'))
config_schema = ProjectConfigSchema()
user_list_update_schema = UserListUpdateSchema(strict=True)
group_import_schema = GroupImportSchema(strict=True)
component_schema = ComponentSchema(strict=True)
components_schema = ComponentSchema(strict=True, many=True)
notification_schema = NotificationSchema(strict=True)
submission_result_schema = SubmissionResultSchema(strict=True)
