import logging
from typing import List

from portal.database.models import Course, Group, User, Project
from portal.database.utils import sql_get_count
from portal.service import general, filters
from portal.service.errors import PortalAPIError, ForbiddenError
from portal.service.general import find_group, find_course, find_role
from portal.service.permissions import check_client
from portal.service.users import find_users

log = logging.getLogger(__name__)



def __count_groups(target, name):
    query = Group.query.filter_by(course=target).filter(
        Group.name.like(name)
        )
    return sql_get_count(query)


def copy_group(source: Group, target: Course, with_users: str):
    """Copies group to a specified course

    Args:
        source(Group): Source group instance
        target(Course): Target course instance
        with_users(str): Whether it should copy with users

    Returns: the new group
    """
    new_name = source.name
    number = __count_groups(target=target, name=new_name)
    new_name += f"{number + 1}"
    new_group = Group(target, new_name)
    if with_users == "with_users":
        for user in source.users:
            user.groups.append(new_group)
    return new_group


def delete_group(group: Group):
    """Deletes group

    Args:
        group(Group): Group instance
    """
    log.info(f"[DELETE] Group {group.id} ({group.name})")
    general.delete_entity(group)


def __set_group_data(group: Group, data: dict) -> Group:
    return general.update_entity(group, data, allowed=['name'])


def update_group(group: Group, data: dict) -> Group:
    """Updates group

    Args:
        group(Group): Group instance
        data(dict): Group data

    Returns(Group): Updated group instance
    """
    log.info(f"[UPDATE] Group {group.name} to {group}.")
    return __set_group_data(group, data)


def create_group(course: Course, data: dict) -> Course:
    """Creates a new group

      Args:
          course(Course): Course instance
          data(dict): Group data

      Returns(Group): Created group instance
      """
    new_group = Group(course=course)
    __set_group_data(new_group, data)
    log.info(f"[CREATE] Group for course {course.id} ({course.codename}): {new_group}")
    return new_group


def update_user_group_membership(group: Group, data: dict):
    """Updates the list of members of the group

    Args:
        group(Group): Group instance
        data(dict): Data
    """
    users_old = set(group.users)
    users_add = data.get('add')
    users_remove = data.get('remove')
    users = set()
    if users_add:
        to_add = find_users(users_add)
        users = users_old.union(to_add)
    if users_remove:
        to_remove = find_users(users_remove)
        users = users_old.difference(to_remove)
    group.users = list(users)
    general.write_entity(group)
    log.info(f"[UPDATE] Users of group {group.name} to {group.users}.")


def find_users_in_group_by_role(course: Course, group: Group, role_id=None) -> List[User]:
    """Finds users in the group by role

    Args:
        course(Course): Course instance
        group(Group): Group instance
        role_id(str): Group id

    Returns(list[User]): List of the users
    """
    role = find_role(course, role_id) if role_id else None
    return group.get_users_based_on_role(role=role)


def add_single_user_to_group(group: Group, user: User) -> Group:
    """Adds a single user to the group

    Args:
        group(Group): Group instance
        user(User): User instance

    Returns(Group): updated group instance
    """
    course = group.course
    if user not in group.users:
        group.users.append(user)
        general.write_entity(group)
        log.info(
            f"[ADD] Added user {user.username} to group {group.name} in course {course.codename}.")
    else:
        log.info(f"[ADD] User {user.username} is already in group {group.name} "
                 f"in course {course.codename}: no change.")
    return group


def remove_single_user_from_group(group: Group, user: User) -> Group:
    """Removes single user from the group

   Args:
       group(Group): Group instance
       user(User): User instance

   Returns(Group): updated group instance
    """
    course = group.course
    try:
        group.users.remove(user)
        general.write_entity(group)
    except ValueError:
        message = f"[REMOVE] Could not remove user " \
                  f"{user.username} from group {group.name} in course " \
                  f"{course.codename}: group does not contain user."
        raise PortalAPIError(400, message=message)
    log.info(f"[REMOVE] User {user.username} from group {group.name}"
             f" in course {course.codename}.")
    return group


def import_group(data: dict, course: Course) -> Group:
    """Imports Group from a course

    Args:
        data(dict): import configuration
        course(Course): Course instance
    """
    source_course = find_course(data['source_course'])
    source_group = find_group(source_course, data['source_group'])
    new_group = copy_group(source_group, course,
                           data['with_users'])
    general.write_entity(new_group)
    log.info(
        f"[IMPORT] Group {source_group.name} from course "
        f"{source_course.codename} to course {course.codename} "
        f"as {new_group.name}.")
    return new_group


def list_groups(course: Course, client):
    if check_client(client=client, course=course, permissions=['view_course_full']):
        return course.groups
    elif check_client(client=client, course=course, permissions=['view_course_limited']):
        return filters.filter_groups_from_course(course=course, user=client)
    raise ForbiddenError(uid=client.id)


def remove_project_from_group(group: Group, project: Project) -> Group:
    """Removes projects from the group
    Args:
        group(Group): Group instance
        project(Project): Project instance
    Returns:
    """

    course = group.course
    try:
        group.projects.remove(project)
        general.write_entity(group)
    except ValueError:
        message = f"[REMOVE] Could not remove project " \
                  f"{project.name} from group {group.name} in course " \
                  f"{course.codename}: group does not contain project."
        raise PortalAPIError(400, message=message)
    log.info(f"[REMOVE] Project {project.name} from group {group.name}"
             f" in course {course.codename}.")
    return group


def add_project_to_group(group: Group, project: Project) -> Group:
    """Adds project to the group
    Args:
        group(Group): Group instance
        project(Project): Project instance

    Returns:
    """
    course = group.course
    if project not in group.projects:
        group.projects.append(project)
        general.write_entity(group)
        log.info(
            f"[ADD] Added project {project.name} to group "
            f"{group.name} in course {course.codename}.")
    else:
        log.info(f"[ADD] User {project.name} is already in group {group.name} "
                 f"in course {course.codename}: no change.")
    return group


def find_projects_in_group(group: Group) -> List[Project]:
    """Lists all the projects in the group
    Args:
        group(Group): Group instance

    Returns:
    """
    return group.projects
